from vantage6.common.client.client_base import ClientBase

from vantage6.client.filter import post_filtering


class PolicySubClient(ClientBase.SubClient):
    """Subclient for the algorithm store policies."""

    @post_filtering(iterable=False)
    def get(self, public: bool = False) -> dict:
        """
        Get the policies of the algorithm store.

        Parameters
        ----------
        public : bool, optional
            If True, only the public policies are returned. If False, all policies
            are returned.
        field: str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the policy. Default is None.
        fields: list[str], optional
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            policy. Default is None.

        Returns
        -------
        list[dict]
            The policies of the algorithm store.
        """
        public_or_not = "/public" if public else ""
        return self.parent.request(
            f"policy{public_or_not}",
            is_for_algorithm_store=True,
        )



from rich.columns import Columns
from rich.console import Console
from rich.panel import Panel
from rich.table import Table
from rich.tree import Tree

from vantage6.client import ClientBase
from vantage6.client.filter import post_filtering
from vantage6.client.rich.table import rich_dataframe_table, rich_session_table


class SessionSubClient(ClientBase.SubClient):
    """Sub client for session management."""

    @post_filtering(iterable=False)
    def get(self, id_: int, display: bool = False) -> dict:
        """
        Get a session by its ID

        Parameters
        ----------
        id_ : int
            The ID of the session
        display : bool, optional
            Print the result in a table. By default False.

        Returns
        -------
        dict
            The session details
        """
        session_metadata = self.parent.request(f"session/{id_}")
        if display:
            rich_session_table(session_metadata)

        return session_metadata

    @post_filtering()
    def list(
        self,
        name: str = None,
        user: int = None,
        collaboration: int = None,
        scope: str = None,
        page: int = 1,
        per_page: int = 20,
        sort: str = None,
        display: bool = False,
    ):
        """
        List of sessions

        Parameters
        ----------
        name : str, optional
            Filter sessions by name. By default None.
        user : int, optional
            Filter sessions by user ID. By default None.
        collaboration : int, optional
            Filter sessions by collaboration ID, overrides the ``collaboration_id``
            of the client. In case both are not set, no filtering is applied.
        scope : str, optional
            Filter sessions by scope, possible values are ``global``, ``collaboration``,
            ``organization`` and ``own``. By default None.
        page : int, optional
            Pagination page. By default 1.
        per_page : int, optional
            Number of items on a single page. By default 20.
        sort : str, optional
            Sort the result by this field. Adding a minus sign in front of the field
            will sort in descending order. By default None.
        display : bool, optional
            Print the result in a table. By default False.

        Returns
        -------
        list[dict]
            Containing session information
        """
        session_metadata = self.parent.request(
            "session",
            params={
                "name": name,
                "user_id": user,
                "collaboration_id": collaboration or self.parent.collaboration_id,
                "scope": scope,
                "sort": sort,
                "page": page,
                "per_page": per_page,
            },
        )

        if display:
            rich_session_table(session_metadata["data"])
            console = Console()
            console.print("page", page)

        return session_metadata

    @post_filtering(iterable=False)
    def update(
        self, id_: int, name: str = None, scope: str = None, display: bool = False
    ):
        """
        Modify a session

        This will update the session with the given ID. Only the fields that are
        provided will be updated.

        Parameters
        ----------
        id_ : int
            The ID of the session
        name : str, optional
            The new name of the session. By default None.
        scope : str, optional
            The new scope of the session. Possible values are ``collaboration``,
            ``organization`` and ``own``. By default None.
        display : bool, optional
            Print the result in a table. By default False.

        Returns
        -------
        dict
            The updated session
        """
        session_metadata = self.parent.request(
            f"session/{id_}",
            method="patch",
            json={"name": name, "scope": scope},
        )

        if display:
            rich_session_table(session_metadata)

        return session_metadata

    @post_filtering(iterable=False)
    def create(
        self,
        scope: str,
        name: str = None,
        collaboration: int = None,
        study: int = None,
        display: bool = False,
    ):
        """
        Create a new session

        This will create an empty session. The session can be populated with one or
        more dataframes.

        Parameters
        ----------
        name: str, optional
            The name of the session. By default None.
        collaboration: int, optional
            The collaboration ID of the session. In case this is not set, the
            collaboration ID of the client is used. When neither is set, the study ID
            needs to be provided.
        study: int, optional
            The study ID of the session. In case this is set, the dataframes in this
            session will be scoped to the study.
        scope: str
            The scope of the session. Possible values are ``collaboration``,
            ``organization`` and ``own``. In case not set, the ``own`` scope is used.
        display: bool, optional
            Print the result in a table. By default False.

        Returns
        -------
        dict
            The created session
        """
        col_id = collaboration or self.parent.collaboration_id
        if not col_id:
            self.parent.log.error("No collaboration ID provided or set in the client.")

        session_metadata = self.parent.request(
            "session",
            method="post",
            json={
                "name": name,
                "collaboration_id": col_id,
                "study_id": study,
                "scope": scope,
            },
        )

        if display:
            rich_session_table(session_metadata)

        return session_metadata

    @post_filtering(iterable=False)
    def delete(self, id_: int, delete_dependents: bool = False):
        """
        Deletes a session

        Parameters
        ----------
        id_ : int
            Id of the session you want to delete
        delete_dependents : bool, optional
            Delete all dependent tasks and dataframes of the session as well. This
            includes tasks dataframes. By default False.
        """
        return self.parent.request(
            f"session/{id_}",
            method="delete",
            params={"delete_dependents": delete_dependents},
        )

    @post_filtering()
    def dataframes(self, id_: int, display: bool = False):
        """
        Get the dataframes of a session

        Parameters
        ----------
        id_ : int
            The ID of the session.
        display : bool, optional
            Print the result in a table. By default False.

        Returns
        -------
        list[dict]
            The dataframes of the session
        """
        dataframes = self.parent.request(f"session/{id_}/dataframe")

        if display:
            rich_dataframe_table(dataframes["data"])

        return dataframes

    def tree(
        self,
        name: str = None,
        user: int = None,
        collaboration: int = None,
        scope: str = None,
        page: int = 1,
        per_page: int = 20,
        sort: str = None,
    ):
        """
        Display a tree of sessions and their dataframes.

        Parameters
        ----------
        name : str, optional
            Filter sessions by name. By default None.
        user : int, optional
            Filter sessions by user ID. By default None.
        collaboration : int, optional
            Filter sessions by collaboration ID, overrides the ``collaboration_id``
            of the client. In case both are not set, no filtering is applied.
        scope : str, optional
            Filter sessions by scope, possible values are ``global``, ``collaboration``,
            ``organization`` and ``own``. By default None.
        page : int, optional
            Pagination page. By default 1.
        per_page : int, optional
            Number of items on a single page. By default 20.
        sort : str, optional
            Sort the result by this field. Adding a minus sign in front of the field
            will sort in descending order. By default None.

        Returns
        -------
        list[dict]
            Containing session information
        """
        session_metadata = self.list(
            name=name,
            user=user,
            collaboration=collaboration,
            scope=scope,
            page=page,
            per_page=per_page,
            sort=sort,
            display=False,
        )

        tree = Tree(Panel("[bold green]Available sessions[/bold green]"))

        console = Console()

        for session in session_metadata["data"]:
            session_base = tree.add(
                f"[bold green]Session{session['id']:─>4} ({session['name']})[/bold green]"
            )
            dataframes = self.dataframes(session["id"])
            session_base.add(
                rich_dataframe_table(dataframes["data"], print_table=False)
            )

        console.print(tree)

    def task_tree(self, session: int | None = None) -> None:
        # TODO FM 02-08-2024: Maybe move this to the task subclient?
        # TODO FM 02-08-2024: This needs to be cleaned

        def color(status):
            color = "green" if status == "completed" else "yellow"
            return f"[{color}]{status}[/{color}]"

        def info_box(header, content):
            return f"[b gray35]{header}[/b gray35]\n[green]{content}"

        def root_task_header(task):
            if task["dataframe"]:
                if task["depends_on"]:
                    type_ = "preprocess"
                else:
                    type_ = "data-extraction"
            else:
                type_ = "compute"

            return [
                info_box("ID", task["id"]),
                info_box("Name", f"{task['name'][:15]}.."),
                info_box("Status", color(task["status"])),
                info_box("Type", type_),
                info_box(
                    "Dataframe",
                    (
                        f"[green]{task['dataframe']['name']}[/green]"
                        if task["dataframe"]
                        else "[gray35]None[/gray35]"
                    ),
                ),
            ]

        session_id = session or self.parent.session_id
        if session_id is None:
            self.parent.log.error("No session ID provided or set in the client.")

        tasks = self.parent.task.list(session=session_id)["data"]
        lookup = {task["id"]: self.parent.task.get(task["id"]) for task in tasks}

        tree = Tree(Panel("[b green]Task dependency tree[/b green]"))
        for task in tasks:
            base = tree.add(
                Panel(
                    Columns(root_task_header(task), expand=True),
                    border_style="white",
                )
            )

            table = Table(border_style="gray35", header_style="gray35")
            table.add_column("ID", style="green")
            table.add_column("Name", style="green")
            table.add_column("Status", style="green")
            table.add_column("Type", style="green")
            table.add_column("Dataframe", style="green")

            for dep_task_id in task["depends_on"]:
                dep_task = lookup[dep_task_id]
                table.add_row(
                    str(dep_task["id"]),
                    dep_task["name"],
                    color(dep_task["status"]),
                    "session-builder" if dep_task["dataframe"] else "compute",
                    (
                        dep_task["dataframe"]["name"]
                        if dep_task["dataframe"]
                        else "None"
                    ),
                )

            if table.rows != []:
                base.add(Panel(table))

        Console().print(tree)



from vantage6.common.enum import StrEnumBase


class LogLevel(StrEnumBase):
    """
    Enum for the different log levels

    Attributes
    ----------
    DEBUG: str
        The debug log level
    INFO: str
        The info log level
    WARN: str
        The warn log level
    ERROR: str
        The error log level
    CRITICAL: str
        The critical log level
    """

    DEBUG = "DEBUG"
    INFO = "INFO"
    WARN = "WARN"
    ERROR = "ERROR"
    CRITICAL = "CRITICAL"



from typing import List

from vantage6.common.client.client_base import ClientBase

from vantage6.client.filter import post_filtering


class StoreRuleSubClient(ClientBase.SubClient):
    """Subclient for the rules from the algorithm store."""

    @post_filtering(iterable=True)
    def list(
        self,
        name: str = None,
        operation: str = None,
        role: int = None,
        user_id: int = None,
        current_user: bool = False,
        page: int = 1,
        per_page: int = 10,
    ) -> List[dict]:
        """
        List algorithms

        Parameters
        ----------
        name : str, optional
            Filter by name (with LIKE operator).
        operation : str, optional
            Filter by operation (view, create, update, delete, or review).
        role : int, optional
            Filter by role id.
        user_id : int, optional
            Filter by user using the user id. If provided together with current_user,
            the current user will be used and user_id will be ignored.
        current_user : bool, optional
            Filter rules for the current user.
        field : str, optional
            Which data field to keep in the result. For instance, "field='name'"
            will only return the name of the rules. Default is None.
        fields : list[str]
            Which data fields to keep in the result. For instance,
            "fields=['name', 'id']" will only return the name and id of the
            rules. Default is None.
        filter_ : tuple[str, str], optional
            Filter the result on key-value pairs. For instance,
            "filter_=('name', 'my_name')" will only return the rules with the
            name 'my_name'. Default is None.
        filters : list[tuple[str, str]], optional
            Filter the result on multiple key-value pairs. For instance,
            "filters=[('name', 'my_name'), ('id', 1)]" will only return the
            rules with the name 'my_name' and id 1. Default is None.
        page : int, optional
            Page number for pagination (default=1)
        per_page : int, optional
            Number of items per page (default=10)

        Returns
        -------
        list[dict]
            List of rules
        """
        params = {
            "name": name,
            "operation": operation,
            "role_id": role,
            "current_user": current_user,
            "page": page,
            "per_page": per_page,
        }
        if user_id:
            params["user_id"] = user_id

        return self.parent.request(
            "rule",
            is_for_algorithm_store=True,
            params=params,
        )



import pandas as pd
from rich.console import Console
from rich.panel import Panel
from rich.table import Table

console = Console()


def _add_columns(table: Table, columns: list[dict]) -> None:
    """
    Add multiple columns to a rich table.

    Parameters
    ----------
    table : Table
        Rich table object
    columns : list[str]
        List of columns names to add to the table.

    Examples
    --------
    This will add columns to a rich table.

    ```python
    table = Table()
    columns = ["column1", "column2"]
    _add_columns(table, columns)
    ```
    """
    for column in columns:
        table.add_column(column["name"])


def rich_dataframe_schema_table(
    dataframe_metadata: dict, print_table: bool = True
) -> None | Table:
    """
    Print the schema of a dataframe in a rich table. The schema includes the
    column name, data type, and the nodes that have this column.

    Parameters
    ----------
    dataframe_metadata : dict
        Metadata of the dataframe. This is the output of `client.dataframe.get()`.
    print_table : bool, optional
        If True, the table will be printed. If False, the table object will be
        returned. Default is True.

    Returns
    -------
    None | Table
        If print_table is False, the table object will be returned.

    Examples
    --------
    This will print a rich table with the schema of a dataframe.

    ```python
    dataframe_metadata = {
        "columns": [
            {
                "name": "column_name",
                "dtype": "int",
                "node_id": 1
            },
        ]
    }
    rich_dataframe_schema_table(dataframe_metadata)
    ```
    """
    columns = dataframe_metadata["columns"]
    if not columns:
        console.print("No columns found in the dataframe.")
        return
    df = pd.DataFrame(columns)

    grouped_df = (
        df.groupby(["name", "dtype"])["node_id"]
        .apply(lambda x: ", ".join(map(str, x)))
        .reset_index()
    )

    table = Table(title="Dataframe Schema")
    table.add_column("Column Name", justify="left", no_wrap=True)
    table.add_column("Data Type", justify="left")
    table.add_column("Nodes", justify="right")

    for _, row in grouped_df.iterrows():
        table.add_row(row["name"], row["dtype"], row["node_id"])

    out = Panel("No columns found in the dataframe.") if table.row_count == 0 else table
    if print_table:
        console.print(out)
    else:
        return out


def rich_session_table(
    session_metadata: dict, print_table: bool = True
) -> None | Table:
    """
    Print a rich table with the metadata of a session. The metadata includes
    the session ID, name, scope, created at, last used at, and ready status.

    Parameters
    ----------
    session_metadata : dict
        Metadata of the session. This is the output of `client.session.get()`.
    print_table : bool, optional
        If True, the table will be printed. If False, the table object will be
        returned. Default is True.

    Returns
    -------
    None | Table
        If print_table is False, the table object will be returned

    Examples
    --------
    This will print a rich table with the metadata of a session.

    ```python
    session_metadata = [
        {
            "id": 1,
            "name": "session_name",
            "scope": "session_scope",
            "created_at": "2021-01-01",
            "last_used_at": "2021-01-02",
            "ready": True
        }
    ]
    rich_session_table(session_metadata)
    ```
    """
    table = Table(title="Sessions")
    table.add_column("ID")
    table.add_column("Name")
    table.add_column("scope")
    table.add_column("created_at")
    table.add_column("last_used_at")
    table.add_column("ready")

    if isinstance(session_metadata, dict):
        session_metadata = [session_metadata]

    for session in session_metadata:
        try:
            table.add_row(
                str(session["id"]),
                session["name"],
                session["scope"],
                session["created_at"],
                session["last_used_at"],
                str(session["ready"]),
            )
        except Exception as e:
            table.add_row("Error", "-", "-", "-", "-", "-")
            console.print(
                f"An error occurred while parsing a row in the session table: {e}"
            )

    out = Panel("No sessions found") if table.row_count == 0 else table
    if print_table:
        console.print(out)
    else:
        return out


def rich_dataframe_table(dataframes: dict, print_table: bool = True) -> None | Table:
    """
    Print a rich table with the metadata of a dataframe. The metadata includes
    the dataframe ID, name, session ID, last session task ID, number of columns,
    and ready status.

    Parameters
    ----------
    dataframes : dict
        Metadata of the dataframe. This is the output of `client.dataframe.get()`.
    print_table : bool, optional
        If True, the table will be printed. If False, the table object will be
        returned. Default is True.

    Returns
    -------
    None | Table
        If print_table is False, the table object will be returned

    Examples
    --------
    This will print a rich table with the metadata of a dataframe.

    ```python
    dataframes = [
        {
            "id": 1,
            "name": "dataframe_name",
            "session": {"id": 1},
            "last_session_task": {"id": 1},
            "columns": [{"name": "column_name", "dtype": "int", "node_id": 1}],
            "ready": True
        }
    ]
    rich_dataframe_table(dataframes)
    ```
    """
    table = Table()
    table.add_column("ID")
    table.add_column("Name")
    table.add_column("Session ID")
    table.add_column("Last Session Task ID")
    table.add_column("No. Columns")
    table.add_column("Ready")

    if isinstance(dataframes, dict):
        dataframes = [dataframes]

    for dataframe in dataframes:
        try:
            table.add_row(
                str(dataframe["id"]),
                dataframe["name"],
                str(dataframe["session"]["id"]),
                str(dataframe["last_session_task"]["id"]),
                str(len(dataframe["columns"])),
                str(dataframe["ready"]),
            )
        except Exception as e:
            table.add_row("Error", "-", "-", "-", "-", "-")
            console.print(
                f"An error occurred while parsing a row in the dataframe table: {e}"
            )

    out = Panel("No dataframes found") if table.row_count == 0 else table
    if print_table:
        console.print(out)
    else:
        return out


def rich_task_table(tasks, print_table=True):
    """
    Print a rich table with the metadata of a task. The metadata includes the
    task ID, name, status, type, and dataframe name.

    Parameters
    ----------
    tasks : dict
        Metadata of the task. This is the output of `client.task.get()`.
    print_table : bool, optional
        If True, the table will be printed. If False, the table object will be
        returned. Default is True.

    Returns
    -------
    None | Table
        If print_table is False, the table object will be returned

    Examples
    --------
    This will print a rich table with the metadata of a task.

    ```python
    tasks = [
        {
            "id": 1,
            "name": "task_name",
            "status": "completed",
            "dataframe": {"name": "dataframe_name"}
        }
    ]
    rich_task_table(tasks)
    ```
    """

    def color(status):
        color = "green" if status == "completed" else "yellow"
        return f"[{color}]{status}[/{color}]"

    table = Table(border_style="gray35", header_style="gray35")
    table.add_column("ID")
    table.add_column("Name")
    table.add_column("Status")
    table.add_column("Type")
    table.add_column("Dataframe")

    for task in tasks:
        table.add_row(
            str(task["id"]),
            task["name"],
            color(task["status"]),
            "session-builder" if task["dataframe"] else "compute",
            task["dataframe"]["name"] if task["dataframe"] else "None",
        )

    out = Panel("No tasks found") if table.row_count == 0 else table
    if print_table:
        console.print(out)
    else:
        return out



from vantage6.common.serialization import serialize

from vantage6.client import ClientBase
from vantage6.client.filter import post_filtering
from vantage6.client.rich.table import rich_dataframe_schema_table, rich_dataframe_table


class DataFrameSubClient(ClientBase.SubClient):
    """Sub client dataframes."""

    @post_filtering(iterable=False)
    def get(self, id_: int, display=False) -> dict:
        """
        Get a dataframe by its ID.

        Parameters
        ----------
        id_ : int
            The ID of the dataframe.
        display : bool, optional
            Whether to print the dataframe details. By default False.

        Returns
        -------
        dict
            The dataframe details.
        """
        df = self.parent.request(f"session/dataframe/{id_}")
        if display:
            rich_dataframe_table(df)
            rich_dataframe_schema_table(df)

        return df

    @post_filtering()
    def list(self, session: int = None, display=False) -> dict:
        """
        List all dataframes.

        Parameters
        ----------
        session : int, optional
            The session ID in which the dataframe is located. When not provided, the
            session ID of the client is used when it is set. In case the session ID is
            not set, an error is printed.
        display : bool, optional
            Whether to print the list of dataframe details. By default False.

        Returns
        -------
        dict
            List of dataframe details.
        """
        session_id = session or self.parent.session_id

        if not session_id:
            self.parent.log.error(
                "No session ID provided and no session ID set in the client."
            )
            return

        dfs = self.parent.request(f"session/{session_id}/dataframe")
        if display:
            rich_dataframe_table(dfs["data"])

        return dfs

    @post_filtering(iterable=False)
    def create(
        self,
        label: str,
        image: str,
        method: str,
        arguments: dict,
        session: int | None = None,
        store: int | None = None,
        name: str | None = None,
        display=False,
    ) -> dict:
        """
        Create a new dataframe in a session.

        Parameters
        ----------
        label : str
            Database label that is specified in the node configuration file.
        image : str
            The name of the image that will be used to retrieve the data from the
            source database.
        method: str
            The method from the algorithm's image to be used for creating the dataframe
        arguments: dict
            Arguments for the algorithm method. The dictionary should contain
            the same keys as the arguments of the algorithm method.
        name: str
            Name that can be used in within the session
        session : int, optional
            The session ID in which the dataframe is located. When not provided, the
            session ID of the client is used when it is set. In case the session ID is
            not set, an error is printed.
        store: int, optional
            The algorithm store where the algorithm image is registered
        display : bool, optional
            Whether to print the dataframe details. By default False.

        Returns
        -------
        dict
            The dataframe details.
        """

        session_id = session or self.parent.session_id
        if not session_id:
            self.parent.log.error(
                "No session ID provided and no session ID set in the client."
            )
            return

        # Get the organizations that are part of the session.
        session = self.parent.request(f"session/{session_id}")
        if not session:
            self.parent.log.error(
                f"An error occurred while fetching session {session_id}"
            )
            return

        if session.get("study"):
            study_id = session["study"]["id"]
            params = {"study": study_id}
        else:
            collaboration_id = session["collaboration"]["id"]
            params = {"collaboration": collaboration_id}

        orgs = self.parent.organization.list(**params)
        organizations = [(o["id"], o["public_key"]) for o in orgs["data"]]

        serialized_arguments = serialize(arguments)

        # Encrypt the input arguments per organization using that organization's
        # public key.
        organization_json_list = []
        for org_id, pub_key in organizations:
            organization_json_list.append(
                {
                    "id": org_id,
                    "arguments": self.parent.cryptor.encrypt_bytes_to_str(
                        serialized_arguments, pub_key
                    ),
                }
            )

        request_payload = {
            "label": label,
            "task": {
                "method": method,
                "image": image,
                "organizations": organization_json_list,
            },
        }

        if name is not None:
            request_payload["name"] = name
        if store is not None:
            request_payload["task"]["store_id"] = store

        df = self.parent.request(
            f"session/{session_id}/dataframe", method="POST", json=request_payload
        )

        if display:
            rich_dataframe_table(df)

        return df

    @post_filtering(iterable=False)
    def preprocess(self, id_: int, image: str, method: str, arguments: dict) -> dict:
        """
        Modify a dataframe in a session.

        Dataframes can be modified by preprocessing them. Preprocessing is handled in
        a sequential manner. In other words, you can add many preprocessing steps to a
        dataframe, and they will be executed one after the other in order of creation.

        The modification will be done after all computation tasks have been executed.
        This is to avoid that a dataframe is modified while it is being used in a
        computation task.

        Parameters
        ----------
        id_: int
            The ID of the dataframe.
        image : str
            The name of the image that will be used to preprocess the dataframe.
        method: str
            Method on the algorithm image to be used to preprocess the dataframe
        arguments: dict
            Arguments for the algorithm method. The dictionary should contain
            the same keys as the arguments of the algorithm method.

        Returns
        -------
        dict
            The dataframe details.
        """

        # Get the organizations that are part of the session.
        dataframe = self.parent.request(f"session/{id_}")
        if not dataframe:
            self.parent.log.error(f"An error occurred while fetching dataframe {id_}")
            return

        if dataframe["study"]:
            params = {"study": dataframe["study"]}
        elif dataframe["collaboration"] and dataframe["collaboration"]["id"]:
            collaboration_id = dataframe["collaboration"]["id"]
            params = {"collaboration": collaboration_id}
        else:
            self.parent.log.error(
                f"No study or collaboration id defined in dataframe {id_}"
            )
            return

        org = self.parent.organization.list(**params)
        organizations = [(o["id"], o["public_key"]) for o in org["data"]]

        # Data will be serialized in JSON.
        serialized_arguments = serialize(arguments)

        # Encrypt the input arguments per organization using that organization's public
        # key.
        organization_json_list = []
        for org_id, pub_key in organizations:
            organization_json_list.append(
                {
                    "id": org_id,
                    "arguments": self.parent.cryptor.encrypt_bytes_to_str(
                        serialized_arguments, pub_key
                    ),
                }
            )

        return self.parent.request(
            f"session/dataframe/{id_}/preprocess",
            method="POST",
            json={
                "dataframe_id": id_,
                "task": {
                    "method": method,
                    "image": image,
                    "organizations": organization_json_list,
                },
            },
        )

    @post_filtering(iterable=False)
    def delete(self, id_: int) -> dict:
        """
        Delete a dataframe.

        Parameters
        ----------
        id_: int
            The ID of the dataframe.
        """

        res = self.parent.request(f"session/dataframe/{id_}", method="DELETE")

        self.parent.log.info(f"--> {res.get('msg')}")
